/*****************************************************************
* IcosahedronRing   By Lukki15
*************************************************************/

innerD = 22.2;
innerR = innerD/2;
g = (1 + sqrt(5)) / 2;

a = sqrt((innerR*innerR*g*g*4)/ (g*g*g*g+(2+g)*(2+g)));
phi = a * ((1 + sqrt(5)) / 2);
$fn=2^11;

module mirrorCopy(x,y,z){
    children();
    mirror([x,y,z]) children();
}

/*****************************************************************
* Icosahedron   By Adam Anderson
* 
* This module allows for the creation of an icosahedron scaled up 
* by a factor determined by the module parameter. 
* The coordinates for the vertices were taken from
* http://www.sacred-geometry.es/?q=en/content/phi-sacred-solids
*************************************************************/
module icosahedron() {
    polyhedron(
        points = [
            [0,-phi, a], [0, phi, a], [0, phi, -a], [0, -phi, -a], [a, 0, phi], [-a, 0, phi], [-a, 0, -phi], 
            [a, 0, -phi], [phi, a, 0], [-phi, a, 0], [-phi, -a, 0], [phi, -a, 0]    
        ],
        faces = [
            [0,5,4], [0,4,11], [11,4,8], [11,8,7], [4,5,1], [4,1,8], [8,1,2], [8,2,7], [1,5,9], [1,9,2], [2,9,6], [2,6,7], [9,5,10], [9,10,6], [6,10,3], [6,3,7], [10,5,0], [10,0,3], [3,0,11], [3,11,7]
        ]
    
    );
}

module icosahedron1() {
    polyhedron(
        points = [
            [0,-phi, a], [0, phi, a], [0, phi, -a], [0, -phi, -a], [a, 0, phi], [-a, 0, phi], [-a, 0, -phi], 
            [a, 0, -phi], [phi, a, 0], [-phi, a, 0], [-phi, -a, 0], [phi, -a, 0]    
        ],
        faces = [
            [0,5,4], [0,4,11], [11,4,8], [4,5,1], [4,1,8], [1,5,9], [9,5,10], [10,5,0], // top part
            [10,0,11], [1,9,8], // sides
            [9,10,11], [8,9,11] // bottom
        ]
    
    );
}

module cutOutInner(){       
    translate([0,0,-centerOffset])
    rotate([90,0,0])
    cylinder(r=r2,h=phi*2, center=true);
}

module outerRing(){
    difference(){
        translate([0,0,-centerOffset])
        rotate([90,0,0])
        cylinder(r=r1,h=a*2, center=true);

        translate([0,0,a*2])
        cube(a*4,true);
    }
}

module cutOutSidesFlat(){
    alpha = atan(a/(phi-a))*1.25;
    
    mirrorCopy(0,1,0)
    translate([0,a,0])
    rotate([alpha-90,0,0])
    translate([-20,0,-40])
    cube(40);
}

module cutOutSides(){
    o = a/4;
    r = (a-o); // (a-o)/2*2
    h = r1+centerOffset;
    mirrorCopy(0,1,0)
    translate([0,a,-h])
    rotate([0,90,0])
    scale([h/r,1,1])
    cylinder(h=3*phi, r=r, center=true);
}

module cutOutEllipse(){
    o = a/4;
    r_a = phi*phi;
    h = r1+centerOffset;
    
    x = o+r_a-a;
    y = h;
    
    // ellipse: x^2/a^2 + y^2/b^2 = 1
    r_b = sqrt(r_a*r_a*y*y/(r_a*r_a-x*x));
    mirrorCopy(0,1,0)
    translate([0,r_a+o,-h])
    rotate([0,90,0])
    scale([r_b/r_a,1,1])
    cylinder(h=3*phi, r=r_a, center=true);
}


centerOffset = a / ((1 + sqrt(5)) / 2);

r1 = sqrt(centerOffset*centerOffset+phi*phi);

c = phi/2;
d = centerOffset + a/2;
r2 = sqrt(c*c+d*d);
echo(r2*2);

translate([0,0,r1+centerOffset])
union(){
difference(){
    icosahedron1();
    cutOutInner();
}
difference(){
    outerRing();
    
    cutOutEllipse();
    //cutOutSides();
    cutOutInner();

}
}